import { default as React } from 'react';
import { Factory, CssKnownProps, CssRule, CssStyleCollection, CssVars } from '@cssfn/core';
import { EventHandler } from '@reusable-ui/hooks';
import { SemanticProps } from '@reusable-ui/semantics';
import { AccessibilityProps } from '@reusable-ui/accessibilities';
import { OutlineableProps } from '@reusable-ui/outlineable';
import { MildableProps } from '@reusable-ui/mildable';
export interface ActivatableVars {
    filterActive: any;
    animActive: any;
    animPassive: any;
    filter: any;
    anim: any;
}
export declare const ifActivated: (styles: CssStyleCollection) => CssRule;
export declare const ifActivating: (styles: CssStyleCollection) => CssRule;
export declare const ifPassivating: (styles: CssStyleCollection) => CssRule;
export declare const ifPassivated: (styles: CssStyleCollection) => CssRule;
export declare const ifActive: (styles: CssStyleCollection) => CssRule;
export declare const ifPassive: (styles: CssStyleCollection) => CssRule;
export declare const ifActivePassivating: (styles: CssStyleCollection) => CssRule;
export interface ActivatableStuff {
    activatableRule: Factory<CssRule>;
    activatableVars: CssVars<ActivatableVars>;
}
export interface ActivatableConfig {
    filterActive?: CssKnownProps['filter'];
    animActive?: CssKnownProps['animation'];
    animPassive?: CssKnownProps['animation'];
}
/**
 * Adds a capability of UI to be highlighted/selected/activated.
 * @param config  A configuration of `activatableRule`.
 * @returns A `ActivatableStuff` represents an activatable state.
 */
export declare const usesActivatable: (config?: ActivatableConfig) => ActivatableStuff;
export interface MarkActiveOptions {
    outlined?: null | OutlineableProps['outlined'];
    mild?: null | MildableProps['mild'];
}
export declare const markActive: (options?: MarkActiveOptions) => CssRule;
export interface ActiveChangeEvent extends Required<Pick<AccessibilityProps, 'active'>> {
}
export interface ActivatableProps extends Partial<Pick<AccessibilityProps, 'active' | 'inheritActive'>> {
}
export declare const enum ActivatableState {
    Passivated = 0,
    Passivating = 1,
    Activating = 2,
    Activated = 3
}
export interface ActivatableApi<TElement extends Element = HTMLElement> {
    active: boolean;
    state: ActivatableState;
    class: string | null;
    props: {
        checked: true;
    } | {
        'aria-checked': true;
    } | {
        'aria-pressed': true;
    } | {
        'aria-selected': true;
    } | null;
    handleAnimationStart: React.AnimationEventHandler<TElement>;
    handleAnimationEnd: React.AnimationEventHandler<TElement>;
    handleAnimationCancel: React.AnimationEventHandler<TElement>;
}
export declare const useActivatable: <TElement extends Element = HTMLElement>(props: ActivatableProps & SemanticProps) => ActivatableApi<TElement>;
export interface ControllableActivatableProps<TActiveChangeEvent extends ActiveChangeEvent = ActiveChangeEvent> extends ActivatableProps {
    onActiveChange?: EventHandler<TActiveChangeEvent>;
}
export interface UncontrollableActivatableProps<TActiveChangeEvent extends ActiveChangeEvent = ActiveChangeEvent> extends ControllableActivatableProps<TActiveChangeEvent> {
    defaultActive?: boolean;
}
export declare const useUncontrollableActivatable: <TActiveChangeEvent extends ActiveChangeEvent = ActiveChangeEvent>(props: UncontrollableActivatableProps<TActiveChangeEvent> & Pick<AccessibilityProps, "enabled" | "readOnly" | "inheritEnabled" | "inheritReadOnly">, changeEventTarget?: (React.RefObject<HTMLInputElement> | null)) => readonly [boolean, React.Dispatch<React.SetStateAction<boolean>>, React.Dispatch<void>];
